//----------------------------------------------------------------------------------
//
// ROSSI-AVR 	RealTime Multitasking Operating System
//

// ROSSI application definitions
#define Xtal 8000
#define SysClock 1		// Scheduler period /ms
#define TaskNbr 5       // Number of tasks  0-4
#define TaskSize 130    // Task stacksize /bytes
#define SigNbr 20       // Number of signals
#define TimerSign 100   // Time synchronization 1/100 sec
#define ResNbr 10       // Number of resources
#define BoxNbr 5        // Number of message boxes
#define BoxLen 3        // Size of box
#define Options 1		// Extra options


// CHECK THIS CONTROLLER DEPENDING DEFINITIONS 
// AT90USB647 / ATmega48/88/168 timer0
#define TOFON 1<<TOV0
#define TOFOFF 255-(1<<TOV0)
#define TOCON 1<<OCF0A
#define TOCOFF 255-(1<<OCF0A)
#define OutputCompare TIMER0_COMPA_vect

/*
// ATmega16/32 timer0
#define TOFON 1<<TOV0
#define TOFOFF 255-(1<<TOV0)
#define TOCON 1<<OCF0
#define TOCOFF 255-(1<<OCF0)
#define TOCFOR 1<<FOC0
#define OutputCompare SIG_OUTPUT_COMPARE0
*/


// default defines
#ifndef Xtal
#define Xtal 6000
#endif
#ifndef SysClock
#define SysClock 1
#endif
#ifndef TaskNbr
#define TaskNbr 5
#endif
#ifndef TaskSize
#define TaskSize 120
#endif
#include <avr/io.h>
#include <avr/interrupt.h>

#define disable asm("cli")
#define enable asm("sei")

#ifndef TCCR0
#define TCCR0 TCCR0B
#endif
#ifndef TIMSK
#define TIMSK TIMSK0
#endif
#ifndef TIFR
#define TIFR TIFR0
#endif
#ifndef OCR0
#define OCR0 OCR0A
#endif


// Schedule function as interrupt
#define Schedule() CallAddr(OutputCompare)


// Initialization of ROSSI: task number =0, clear other tasks
//	clear boxes and resources, init timer0
void InitRossi(void);
//  Create a task with number nro and start it
void CreateTask(void *tadr, int nro);

#if defined (SigNbr)
// Stay to wait signal 'nro'
void WaitSign(int nro);
// Send signal 'nro' e.g move waiting tasks to ready-to-run
void SendSign(int nro);
// Send signal 'nro' e.g move waiting tasks to ready-to-run
void PutSign(int nro);
#endif

#if defined (ResNbr)
// Reserve resource 'nro' or stay waiting release
void ReserveRes(int nro);
// Release resource 'nro'
void FreeRes(int nro);
// Ask reaource state
char AskRes(int nro);
#endif

#if defined (BoxNbr)
// Wait data from box 'nro'
int WaitBox(int nro);
// Ask box state
char AskBox(int nro);
// Send data 'dat' to box 'nro'
char SendBox(int nro,int dat);
// Send data 'dat' to box 'nro', interrupts disabled
char PutBox(int nro,int dat);
// Empty box 'nro'
void ClrBox(int nro);
#endif

#if defined (Options)
// Check, is task exchange executed
char Scheduled(void);
// Move task 'nro' to ready-to-run state
void RunTask(int nro);
// End task 'nro' execution
void StopTask(int nro);
// Ask task number
unsigned char MyTask(void);
#if defined (SysClock)
// Disable scheduler
void Lock(void);
// Enable sheduler
void Unlock(void);
#endif
#endif
